#!/bin/sh

#
# Z2 flash script v0.01 alpha 0
# Copyright (C) 2010 Marek Vasut <marek.vasut@gmail.com>
#
# This is under GPL v3
#
# Modified by mozzwald http://www.mozzwald.com

# Make sure only root can run our script
if [ "$(id -u)" != "0" ]; then
   echo "This script must be run as root" 1>&2
   exit 1
fi

echo "************************************************"
echo "        Zipit Z2 u-boot flasher - RESTORE"
echo "************************************************"
echo ""
echo "WARNING: THIS COULD BRICK YOUR ZIPIT!"
echo "No one but YOU is responsible if something"
echo "goes wrong. "
echo ""

# Check for directory argument and if it exists
if [ -z "$1" ]; then
	echo "Restore directory not provided!"
	echo "Usage: restore.sh /path/to/folder"
	exit 1
fi
if [ -e "$1" ]; then
	echo "Restore directory found"
else
	echo "Restore directory not found!"
	echo "You provided $1 which doesn't exist"
	echo "exiting.."
	exit 1
fi

image=image.img
partlist=plist.lst
flashdefs="$1/restore.def"

echo "Preparing flash image.."
if [ -e "$image" ] ; then
	rm -f "$image";
fi
touch "$image"

if [ -e "$partlist" ] ; then
	rm -f "$partlist";
fi
touch "$partlist"

# create the complete image file with proper spacing
offset=0
while read file start md5; do
	if ! [ -e "$1/$file" ] ; then
		echo "File \"$1/$file\" not found, aborting!"
		exit 1;
	fi

	# check md5sum
	if [ -z "$md5" ] ; then
		echo "md5sum for \"$1/$file\" not found, aborting!"
		exit 2;
	elif [ "$md5" != "`md5sum $1/$file | cut -d " " -f 1`" ] ; then
		echo "md5sum for \"$1/$file\" does not match, aborting!"
		exit 3;
	fi

	# calculate empty space between parts
	pad=`expr "$start" - "$offset"`

	if [ "$pad" -lt 0 ] ; then
		echo "Image components overlap, aborting!"
		exit 2;
	elif [ "$pad" -gt 0 ] ; then
		# write empty space to image file
		dd if=/dev/zero bs="$pad" count=1 >> "$image" 2>/dev/null;
	fi

	# write part to image file
	cat "$1/$file" >> "$image"
	# calculate next offset
	offset=`wc -c "$image" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`;
	#echo "Offset: $offset"
	md5="\0"
done < "$flashdefs"
echo "$image ready for flashing."

# fix mtdblock partitions and create partition list
flashsz=0
#for i in `ls -1 /dev/ | egrep "mtd[[:digit:]]+"` ; do
while read line; do
        i=`echo $line | grep mtdblock | cut -d " " -f 4`
       	if [ -n "$i" ]; then
		# fix the mtdblock
		echo "Fixing /dev/$i"
		blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
		if [ -e "/dev/$i" ]; then
			rm "/dev/$i"
		fi
		mknod "/dev/$i" b 31 "$blocknum"
		# write the partition list
		partsz=`wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
		echo "/dev/$i $partsz" >> "$partlist"
		flashsz=`expr "$flashsz" + "$partsz"`
		mtdcount=`expr "$blocknum" + 1`
		#echo "$blocknum PartSize: $partsz FlashSize: $flashsz"
	fi
done </proc/partitions

if [ "$offset" -gt "$flashsz" ] ; then
	echo "Image too big for flash, aborting!"
	echo "Offset: $offset FlashSize: $flashsz"
	exit 3;
fi

if [ -z "$partlist" ] ; then
	echo "Partition list empty, aborting!"
	exit 4;
fi

echo "Make sure your Zipit is fully charged and"
echo "you have the AC Adapter plugged in. "

agree=" "
read -p "Last chance before flashing. Are you sure? (yes/no): " agree
if [ "$agree" != "yes" ]; then
  echo "You chose not to continue."
  echo "Did you type \"yes\" correctly?"
  echo "exiting..."
  exit 5
fi

echo "Partitons to be over-written: $mtdcount"
echo "Starting flashing procedure, do NOT POWER OFF!"

offset=0
while read part size ; do
	echo "Writing to $part"
	dd if="$image" of="$part" bs=1 skip="$offset" count="$size"
	offset=`expr "$offset" + "$size"`;
done < "$partlist"

rm -f "$partlist" "$image"

echo "Flashing complete!"
agree=0
read -p "Are you ready to halt the system? (yes/no): " agree
if [ "$agree" != "yes" ]; then
  echo "When you are finished with the shell, be sure to"
  echo "unplug the AC Power and remove the battery for a"
  echo "moment."
  echo ""
  echo "Re-attach the battery/AC and the system should boot."
  echo "Exiting to shell..."
  exit 0
fi

echo "After the system is halted, unplug the AC Power"
echo "and remove the battery for a moment. After you"
echo "re-attach the battery/AC, the system should boot."
echo "Shutting down the system..."
sleep 5
halt
